function [voxels regions]=peak_nii(image,varstruct)
%%
% peak_nii will write out the maximum T (or F) of the local maxima that are
% not closer than a specified separation distance.  
% SPM=0: Those that are closer are collapsed based on the COG using number 
%   of voxels at each collapsing point. The maximum T 
%   (or F) is retained. This program should be similar to peak_4dfp in use at
%   WashU (although I haven't seen their code).
% SPM=1: Eliminates the peaks closer than a specified distance to mimic
%   result tables.
%
% INPUTS:
% image string required. This should be a nii or img file.
% varstruct is either a .mat file or a pre-load structure with the
% following fields:
%           out: output prefix, default is to define using imagefile
%          sign: 'pos' or 'neg', default is 'pos'
%          type: statistic type, 'T' or 'F' or 'none'
%      voxlimit: number of peak voxels in image
%    separation: distance to collapse or eliminate peaks
%           SPM: 0 or 1, see above for details
%          conn: connectivity radius, either 6,18, or 26
%       cluster: cluster extent threshold in voxels
%          mask: optional to mask your data
%           df1: numerator degrees of freedom for T/F-test
%           df2: denominator degrees of freedom for F-test
%       nearest: 0 or 1, 0 for leaving some clusters/peaks undefined, 1 for finding the
%       nearest label
%         label: optional to label clusters, options are 'aal_MNI_V4' or
%         'Nitschke_Lab'; more will be added in the future.
%        thresh: T/F statistic or p-value to threshold the data
%          mask: optional mask file for the data.
%
% OUTPUTS:
%   voxels  -- table of peaks
%       col. 1 - Cluster size
%       col. 2 - T/F-statistic
%       col. 3 - X coordinate
%       col. 4 - Y coordinate
%       col. 5 - Z coordinate
%       col. 6 - number of peaks collapsed
%       col. 7 - original cluster number
%       col. 8 - sorted cluster number
%       col. 9 - region number (optional)
%   regions -- region of each peak -- optional
%
% EXAMPLE: voxels=peak_nii('imagename',varstruct)
%
%   peak_nii.v2 -- Last modified on 11/26/2010 by Donald G. McLaren, PhD
%   (mclaren@nmr.mgh.harvard.edu)
%   Wisconsin Alzheimer's Disease Research Center - Imaging Core, Univ. of
%   Wisconsin - Madison
%   Neuroscience Training Program and Department of Medicine, Univ. of
%   Wisconsin - Madison
%   GRECC, William S. Middleton Memorial Veteren's Hospital, Madison, WI
%   GRECC, Bedford VAMC
%   Department of Neurology, Massachusetts General Hospital and Havard
%   Medical School


%% Check inputs
if exist(image,'file')==2
    I1=spm_vol(image);
    infoI1=I1;
    [I1,voxelcoord]=spm_read_vols(I1);
    if nansum(nansum(nansum(abs(I1))))==0
        error(['Error: ' image ' is all zeros or all NaNs'])        
    end
else
    error(['File ' image ' does not exist'])
end
try
if exist(varstruct,'file')==2
    varstruct=load(varstruct);
end
end

invar=peak_nii_inputs(varstruct,infoI1.fname,nargout);

%% Read in data and mask (if available)
disp(['Threshold is:' num2str(invar.thresh)])

if strcmpi(invar.sign,'neg')
    I1=-1.*I1;
end

if ~isempty(invar.mask)
    I2=spm_vol(invar.mask);
    infoI2=I2;
    if infoI1.mat==infoI2.mat
        if infoI1.dim==infoI2.dim
            I2=spm_read_vols(I2);
        else
            Vi(1)=infoI1; Vi(2)=infoI2;
            Vo=Vi(1); Vo.pinfo=Vi(2).pinfo; Vo.fname='tmp.nii';
            Vo=spm_imcalc(Vi,Vo,'i2');
            I2=spm_read_vols(spm_vol('tmp.nii'));
        end
    else
        Vi(1)=infoI1; Vi(2)=infoI2;
        Vo=Vi(1); Vo.pinfo=Vi(2).pinfo; Vo.fname='tmp.nii';
        spm_imcalc(Vi,Vo,'i2',{[],[],0});
        I2=spm_read_vols(spm_vol('tmp.nii'));
    end
            
    I2=(I2>0)+(I2<0); % Ensures that the mask is binary
    I=I1.*I2; % Mask Dataset
    %% Mask Region file
    try
        invar.label.rf.img=invar.label.rf.img.*(I2~=0);
    catch
        Vi(1)=infoI1; Vi(2)=invar.label.rf.hdr;
        Vo=Vi(1); Vo.pinfo=Vi(2).pinfo; Vo.fname='tmp.nii';
        spm_imcalc(Vi,Vo,'(i1~=0).*i2',{[],[],0});
        invar.label.rf.hdr=spm_vol('tmp.nii');
        [invar.label.rf.img, invar.label.rf.XYZmm]=spm_read_vols(invar.label.rf.hdr);
        invar.label.rf.img=invar.label.rf.img.*(I2>0); % Mask Region File
    end
else
    I=I1;
end

%% Program begins here
% Find significant voxels
ind=find(I>invar.thresh);
if isempty(ind)
    error(['NO MAXIMA ABOVE ' num2str(invar.thresh) '.'])
else
   [L(1,:),L(2,:),L(3,:)]=ind2sub(infoI1.dim,ind);
end

% Cluster signficant voxels
A=peakcluster(L,invar.conn,infoI1); % A is the cluster of each voxel
A=transpose(A);
n=hist(A,1:max(A));
for ii=1:size(A,1)
    if n(A(ii))<invar.cluster % removes clusters smaller than extent threshold
        A(ii,1:2)=NaN;
    else
        A(ii,1:2)=[n(A(ii)) A(ii,1)];
    end
end

% Combine A (cluster labels) and L (voxel indicies)
L=L';
A(:,3:5)=L(:,1:3);
% Remove voxels that are in small clusters
A(any(isnan(A),2),:) = [];

% Save clusters
T=peakcluster(transpose(A(:,3:5)),invar.conn,infoI1,invar.out);
A(:,2)=T(:,1); clear T
% Save significant data
Iclust=spm_read_vols(spm_vol([invar.out '_clusters.nii']));
Ithresh=(I>invar.thresh).*(Iclust>0);
out=infoI1;
out.fname=[invar.out '_thresh' num2str(invar.thresh) '_cluster' num2str(invar.cluster) '.nii'];
out.descrip=['Thresholded Map @ thresh ' num2str(invar.thresh) ' and cluster extent ' num2str(invar.cluster)];
spm_write_vol(out,Ithresh);


% Find all peaks, only look at current cluster to determine the peak
Ic=zeros(infoI1.dim(1),infoI1.dim(2),infoI1.dim(3),max(A(:,2)));
for ii=1:max(A(:,2))
    Ic(:,:,:,ii)=I.*(Iclust==ii);
end
N=0;
voxelsT=zeros(size(A,1),7);
for ii=1:size(A,1)
    if A(ii,3)==1 || A(ii,4)==1 || A(ii,5)==1 || A(ii,3)==size(Ic,1) || A(ii,4)==size(Ic,2) || A(ii,5)==size(Ic,3)
    else
        if I(A(ii,3),A(ii,4),A(ii,5))==max(max(max(Ic(A(ii,3)-1:A(ii,3)+1,A(ii,4)-1:A(ii,4)+1,A(ii,5)-1:A(ii,5)+1,A(ii,2)))))
            N=N+1;
            voxind=sub2ind(infoI1.dim,A(ii,3),A(ii,4),A(ii,5));
            voxelsT(N,1)=A(ii,1);
            voxelsT(N,2)=I(voxind);
            voxelsT(N,3)=voxelcoord(1,voxind);
            voxelsT(N,4)=voxelcoord(2,voxind);
            voxelsT(N,5)=voxelcoord(3,voxind);
            voxelsT(N,6)=1;
            voxelsT(N,7)=A(ii,2);
        end
    end
end

%Remove empty rows
voxelsT=voxelsT(any(voxelsT'),:);
if isempty(voxelsT)
    voxels=[];
    return
end
%Check number of peaks
if size(voxelsT,1)>invar.voxlimit
    voxelsT=sortrows(voxelsT,-2);
    voxelsT=voxelsT(1:invar.voxlimit,:); % Limit peak voxels to invar.voxlimit
end

% Sort table by cluster w/ max T then by T value within cluster (negative
% data was inverted at beginning, so we are always looking for the max).
uniqclust=unique(voxelsT(:,7));
maxT=zeros(length(uniqclust),2);
for ii=1:length(uniqclust)
    maxT(ii,1)=uniqclust(ii);
    maxT(ii,2)=max(voxelsT(voxelsT(:,7)==uniqclust(ii),1));
end
maxT=sortrows(maxT,-2);
for ii=1:size(maxT,1)
    voxelsT(voxelsT(:,7)==maxT(ii,1),8)=ii;
end
voxelsT=sortrows(voxelsT,[8 -2]);
[cluster,uniq,ind]=unique(voxelsT(:,8)); % get rows of each cluster

%Collapse or elimintate peaks closer than a specified distance
voxelsF=zeros(size(voxelsT,1),size(voxelsT,2));
nn=[1 zeros(1,length(cluster)-1)];
for numclust=1:length(cluster)
    Distance=eps;
    voxelsC=voxelsT(ind==numclust,:);
    while min(min(Distance(Distance>0)))<invar.separation
            [voxelsC,Distance]=vox_distance(voxelsC);
            minD=min(min(Distance(Distance>0)));
            if minD<invar.separation
               min_ind=find(Distance==(min(min(Distance(Distance>0)))));
               [ii,jj]=ind2sub(size(Distance),min_ind(1));
               if invar.SPM==1
                    voxelsC(ii,:)=NaN; % elimate peak
               else
                    voxelsC(jj,1)=voxelsC(jj,1);
                    voxelsC(jj,2)=voxelsC(jj,2);
                    voxelsC(jj,3)=((voxelsC(jj,3).*voxelsC(jj,6))+(voxelsC(ii,3).*voxelsC(ii,6)))/(voxelsC(jj,6)+voxelsC(ii,6)); % avg coordinate
                    voxelsC(jj,4)=((voxelsC(jj,4).*voxelsC(jj,6))+(voxelsC(ii,4).*voxelsC(ii,6)))/(voxelsC(jj,6)+voxelsC(ii,6)); % avg coordinate
                    voxelsC(jj,5)=((voxelsC(jj,5).*voxelsC(jj,6))+(voxelsC(ii,5).*voxelsC(ii,6)))/(voxelsC(jj,6)+voxelsC(ii,6)); % avg coordinate
                    voxelsC(jj,6)=voxelsC(jj,6)+voxelsC(ii,6);
                    voxelsC(jj,7)=voxelsC(jj,7);
                    voxelsC(jj,8)=voxelsC(jj,8);
                    voxelsC(ii,:)=NaN; % eliminate second peak
               end
               voxelsC(any(isnan(voxelsC),2),:) = [];
            end
    end
    try
        nn(numclust+1)=nn(numclust)+size(voxelsC,1);
    end
    voxelsF(nn(numclust):nn(numclust)+size(voxelsC,1)-1,:)=voxelsC;
end
voxelsT=voxelsF(any(voxelsF'),:);
clear voxelsF voxelsC nn

% Label Peaks
if ~isempty(invar.label.source)
    regions=cell(size(voxelsT,1),2);
    for i=1:size(voxelsT,1)
             [regions{i,1} regions{i,2}]=regionname(voxelsT(i,:),invar.label.rf,invar.label.ROI,invar.label.ROInames,invar.label.nearest);
    end
end

% Modify T-values for negative
if strmatch(invar.sign,'neg')
    voxelsT(:,1)=-1*voxelsT(:,1);
end

% Save Output
[path,file,ext]=fileparts(invar.out);
try
    save([path filesep 'Peak_' file],'regions', 'voxelsT')
catch
    save([path filesep 'Peak_' file],'voxelsT')
end
try
    voxels=[voxelsT [regions{:,1}]'];
catch
    voxels=voxelsT;
end
return

%% Embedded functions

%% Vox_distance
function [N,Distance] = vox_distance(voxelsT)
% vox_distance compute the distance between local maxima in an image
% The input is expected to be an N-M matrix with columns 2,3,4 being X,Y,Z
% coordinates
%
% pdist is only available with Statistics Toolbox in recent versions of
% MATLAB, thus, the slower code is secondary if the toolbox is unavailable.
% Speed difference is dependent on cluster sizes, 3x at 1000 peaks.
N=sortrows(voxelsT,-1);
try
    Distance=squareform(pdist(N(:,3:5)));
catch
    Distance = zeros(size(N,1),size(N,1));
    for ii = 1:size(N,1);
        TmpD = zeros(size(N,1),3);
        for kk = 1:3;
            TmpD(:,kk) = (N(:,kk+2)-N(ii,kk+2)).^2;
        end
        TmpD = sqrt(sum(TmpD,2));
        Distance(:,ii) = TmpD;
    end
end
%Distance=zeros(length(N(:,1)),length(N(:,1)))*NaN;
%for ii=1:length(N(:,1))
%    for jj=ii+1:length(N(:,1))
%           Distance(ii,jj)=((N(jj,2)-N(ii,2)).^2)+((N(jj,3)-N(ii,3)).^2)+((N(jj,4)-N(ii,4)).^2);
%    end
%end
return

%% Peakcluster
function A=peakcluster(L,conn,infoI1,out)
dim = infoI1.dim;
vol = zeros(dim(1),dim(2),dim(3));
indx = sub2ind(dim,L(1,:)',L(2,:)',L(3,:)');
vol(indx) = 1;
[cci,num] = spm_bwlabel(vol,conn);
A = cci(indx');

if nargin==4
    infoI1.fname=[out '_clusters.nii'];
    infoI1.descrip='clusters';
    infoI1.pinfo=[1 0 0]';
    A=transpose(A);
    L=transpose(L);
    A(:,2:4)=L(:,1:3);
    vol=zeros(dim(1),dim(2),dim(3));
    for ii=1:length(A)
        vol(A(ii,2),A(ii,3),A(ii,4))=A(ii,1);
    end
    spm_write_vol(infoI1,vol);
end
return

%% Regionname
function [ROInum ROIname]=regionname(voxel,rf,ROI,ROInames,nearest)
if nearest==0
        [xyz,ii] = spm_XYZreg('NearestXYZ',voxel(3:5),rf.XYZmm); % use all voxels
else    
        nz_ind=find(rf.img>0);
        [xyz,j] = spm_XYZreg('NearestXYZ',voxel(3:5),rf.XYZmm(:,nz_ind)); % use only voxels with a region greater than 0.
        ii=nz_ind(j);
        [junk,D]=vox_distance([0 0 xyz';voxel(1:5)]); % need to pad xyz, voxel is made to be 1x4 with 2-4 being coordianates
        if D(1,2)>5 % further than 5 mm from region
            ii=0;
        end
end
ROInum=rf.img(ii);
if ROInum~=0
       ROIind=find([ROI.ID]==ROInum);
       ROIname=ROInames{ROIind};
else
       ROIname='undefined';
end
return
